/* *****************************************************************************************************
 * SOFTWARE COPYRIGHT NOTICE 
 *  
 * (c) 2016 Microchip Technology Inc. and its subsidiaries.  
 * You may use this software and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR 
 * STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 * MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, 
 * COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 * 
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL OR CONSEQUENTIAL 
 * LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN 
 * IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST 
 * EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR 
 * THIS SOFTWARE.
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS. 
 * ****************************************************************************************************/


#ifndef _TABLE_NTC_8BIT_H_
#define _TABLE_NTC_8BIT_H_


/*@@tab_ntc8b.h
 * *****************************************************************************
 * Summary:
 * Provides a scaled lookup table for a 10k NTC
 *
 * Description:
 * This look-up table has been created for a 10k NTC-type. This NTC is connected
 * in series to a 10k reference resistor to form a voltage divider. The bias voltage
 * of this voltage divider is VDD of the controller.
 * Each index of this look-up table provides a temperature in [C] for each 
 * value of a 10-bit ADC (Vref = VDD)
 *
 * The temperatures given here are scaled to single digit C fitting into a 
 * signed 8bit integer number.
 * 
 * The upper two values (ADC-reading 1022 and 1023) will trip an artificial temperature 
 * value of -100C. This Out-Of-Scope value can be used to detect broken connections 
 * between LED driver and LED load.
 * 
 * *****************************************************************************
 */

#include <stdint.h>

extern const int8_t tab_ntc[];


#endif
