//01/12/2016 19:11:10/V2.204
//19/11/2015 15:54:17/V2.202
//04/05/2015 14:23:54

/* *****************************************************************************************************
 * SOFTWARE COPYRIGHT NOTICE 
 *  
 * (c) 2016 Microchip Technology Inc. and its subsidiaries.  
 * You may use this software and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR 
 * STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 * MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, 
 * COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 * 
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL OR CONSEQUENTIAL 
 * LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN 
 * IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST 
 * EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR 
 * THIS SOFTWARE.
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS. 
 * ****************************************************************************************************/


#ifndef APPLICATION_LAYER_TYPES_H
#define APPLICATION_LAYER_TYPES_H

#include <xc.h>
#include <stdint.h>

typedef struct
{
	uint8_t switch_pre_state;
	uint8_t switch_counter;
	uint8_t switch_counter_threshold;
} switch_controller_t;

typedef struct
{
	volatile uint16_t trip; // Fault Condition Trip Point Threshold
	volatile uint16_t release; // Fault Condition Release Point Threshold
	volatile uint8_t counter; // Fault Condition Occurrence Counter
	volatile uint8_t threshold; // Fault Condition Occurrence Counter Shut-Down Threshold
} fault_settings_t;

typedef struct
{
    union 
    {
        struct
        {
            volatile unsigned flt_fault_detect : 1; // Temporary fault flag 
            volatile unsigned flt_uvlo_shutdown : 1; // Under Voltage Lock Out Condition occurred
            volatile unsigned flt_ovlo_shutdown : 1; // Over Voltage Shut Down Condition occurred
            volatile unsigned flt_ovp_shutdown : 1; // Output Over Voltage Shut Down occurred
            volatile unsigned flt_ot_warning : 1; // Over Temperature Warning is active
            volatile unsigned flt_otp_shutdown : 1; // Over Temperature Shut Down occurred
            volatile unsigned flt_open_circuit : 1; // Open connection between LED driver and load
            volatile unsigned : 1; // (reserved))
        };
        volatile uint8_t status; 
    };
} fault_status_t;

typedef struct
{
	volatile fault_settings_t fltSettingsUVLO; // Under Voltage Lock Out Settings
	volatile fault_settings_t fltSettingsOVLO; // Over Voltage Lock Out Settings
	volatile fault_settings_t fltSettingsOVP; // Output Over Voltage Protection Settings
	volatile fault_settings_t fltSettingsOTW; // Over Temperature Warning Settings
	volatile fault_settings_t fltSettingsOTP; // Over Temperature Protection Settings

	volatile fault_status_t status; // Global Fault Status Flags
	
	volatile uint8_t fault_recovery_counter; // Global Fault Recovery Counter
	volatile uint8_t fault_recovery_counter_thshld; // Global Fault Recovery Counter Threshold

} fault_control_t;

typedef struct
{
    union 
    {
        struct
        {
            volatile unsigned global_fault : 1; // This global flag is set when the converter is shut down due to a critical fault condition
            volatile unsigned temp_derating_active : 1; // Global flag indicating active temperature derating
            volatile unsigned startup : 1; // This flag is set until the converter has passed startup phase
            volatile unsigned phase_in :1; // Flag for soft transitions from one current setting into another (e.g. soft-start)
            volatile unsigned enabled : 1; // Global enable flag
            volatile unsigned : 3; // (reserved))
        };
        volatile uint8_t status;   
    };
} converter_status_t;

typedef struct
{
	/* Monitoring results */
	volatile uint16_t input_voltage; // Most recent input voltage sample
	volatile uint16_t output_voltage; // Most recent output voltage sample
////	volatile uint16_t output_current; // Most recent output current sample
////	volatile int8_t led_temperature; // Most recent LED load temperature sample in C

	/* System property detection results */
////	volatile uint16_t led_bin_class; // Most recent LED BIN class voltage sample
////	volatile uint16_t led_bin_current; // Most recent LED BIN class voltage sample
////
////	/* Fixed set point */
////	volatile uint16_t led_dim_step; // Sets/holds the current dimming step
////	volatile uint16_t led_dim_ratio; // Sets/holds the current dimming ratio
	volatile uint16_t led_forward_current;	// Sets/holds the LED forward current set value
	
	/* Status information */
	volatile converter_status_t status; // Global status flags

} converter_t;

typedef struct
{
    volatile uint8_t send[10];
    volatile uint8_t counter;
    volatile uint8_t maxCount;
    volatile uint8_t SendTaskCounter;
} UARTSend_t;

typedef struct 
{
    volatile unsigned On                 : 1; //LEDs are on/off
    volatile unsigned Ramp               : 1; //LEDs ramp uo/down
    volatile unsigned HazardLight        : 1; //simulation hazard light
    volatile unsigned RampDown           : 1; //simulation running light up/down
    volatile unsigned RampPause          : 1; //simulation pause
    volatile unsigned Start              : 1; //start of ramp
    volatile unsigned BlinkOn            : 1; //LIN message
    volatile unsigned ScenarioReady      : 1; //scenario is completed
    volatile uint16_t RampDACCounter;
    volatile uint8_t GlobalTestCounter;
    volatile uint16_t RisingSteps;
    volatile uint16_t VoutputVoltage;
    volatile uint8_t FunctionMode;
    volatile uint8_t FunctionModePrevious;
    volatile uint8_t FunctionModeSet;
    
} RampControl_t;

#endif	/* APPLICATION_LAYER_TYPES_H */



