//19/02/2016 19:24:44/V3.1 Animation only

//-----------------------------------------------------------------------------
//  2015 Microchip Technology Inc. and its subsidiaries.  
//You may use this software and any derivatives exclusively with Microchip products. 
//  
//THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, 
//IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF 
//NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS 
//INTERACTION WITH MICROCHIP PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR 
//USE IN ANY APPLICATION. 
//
//IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL 
//OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED 
//TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE 
//POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWED BY 
//LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE 
//WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO 
//MICROCHIP FOR THIS SOFTWARE.
//
//MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS. 
//-----------------------------------------------------------------------------
#include <xc.h>
#include <stdint.h>

#include "globals.h"
#include "mcc.h"

//------------------------------------------------------------------------------
// Ramp
//                   ---------------------
//                 -                      |
//               -                        |
//             -                          |
//           -                            |
//         -                              |
//       -                                |                   -
//     -                                  |                 -              
//   -                                    |               -
//  |                                     |              |
//  |                                     ---------------|
//  <---------------><-------------------><------------->
//     (.Ramp !.On )     (.On  !.Ramp)     (!.Ramp  !.On)
//------------------------------------------------------------------------------
void RampControll (RampControl_t *VRampControl_)
{
  //ramp up
  if ((VRampControl_->Ramp) && (!VRampControl_->On))
  {
    if(VRampControl_->Start)
    {
      StartPSU();
      VRampControl_->Start = 0;
      VRampControl_->ScenarioReady = 0;
////      VRampControl.VoutputVoltage = RampStartVoltage;
////      VRampControl.RisingSteps = (uint8_t)RampRisingStep;
    }  
 
    if (VRampControl_->RampDACCounter < MsTicksRampTime)
    {
////      VRampControl_->VoutputVoltage = VRampControl_->VoutputVoltage + VRampControl_->RisingSteps;
////      VRampControl_->VoutputVoltage = VRampControl_->VoutputVoltage + 3;
      VRampControl_->VoutputVoltage = RampDACValues[VRampControl_->RampDACCounter];
      DAC1_Load10bitInputData(VRampControl_->VoutputVoltage);
      
      
      VRampControl_->RampDACCounter++;
    }
    else 
    {
      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 0;
      VRampControl_->On = 1;
    }    
  }
  
  //all LEDs on after ramp up
  if ((!VRampControl_->Ramp) && (VRampControl_->On))
  {
    if(VRampControl_->RampDACCounter++ > MsTicksOnTime)
    {
      DAC1_Load10bitInputData(0); //added FTx
      StopPSU();
 
      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 0;
      VRampControl_->On = 0;
    }
  }
  
  //all LEDs off 
  if ((!VRampControl_->Ramp) && (!VRampControl_->On))
  {
    if(VRampControl_->RampDACCounter++ > MsTicksOffTime)
    {
      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 1;
      VRampControl_->On = 0;
      
      VRampControl_->GlobalTestCounter++;
      
      VRampControl_->Start = 1;

      VRampControl_->ScenarioReady = 1;
    } 
  }
}
//------------------------------------------------------------------------------
// Hazard
//  --------------------------------------
//  |                                     |
//  |                                     |
//  |                                     |
//  |                                     |
//  |                                     |
//  |                                     |                   
//  |                                     |                               
//  |                                     |               
//  |                                     |              |
//  |                                     ---------------|
//  <---------------><-------------------><------------->
//     (.Ramp !.On )     (.On  !.Ramp)     (!.Ramp  !.On)
//------------------------------------------------------------------------------
void HazardControll (RampControl_t *VRampControl_)
{ 
  uint8_t i_;
  
  //all LEDs on with 3 small steps from min to max
  if ((VRampControl_->Ramp) && (!VRampControl_->On))
  {
    if(VRampControl_->Start)
    {
      StartPSU();
      VRampControl_->Start = 0;
      VRampControl.VoutputVoltage = 0;
      VRampControl.RisingSteps = (uint16_t)RampRisingEdge;
      
      for(i_=0;i_<200;i_++)
      {
        DAC1_Load10bitInputData(RampDACValues[i_]);
      }  
    }  
     
    if (VRampControl_->RampDACCounter < MsTicksRampTime)
    {
//////      if(VRampControl_->RampDACCounter < 11)
////      if(VRampControl_->RampDACCounter < RisingEdgeCount+1)
////      {  
////        VRampControl_->VoutputVoltage = VRampControl_->VoutputVoltage + VRampControl.RisingSteps;
////////        VRampControl_->VoutputVoltage = RampDACValues[VRampControl_->RampDACCounter];
////      }  
////      else
////      {
////        VRampControl.VoutputVoltage = RampDACValues[199];
////      }
////      DAC1_Load10bitInputData(VRampControl_->VoutputVoltage);
      VRampControl_->RampDACCounter++;
    }
    else 
    {
      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 0;
      VRampControl_->On = 1;
    }    
  }
  
  //all LEDs on 
  if ((!VRampControl_->Ramp) && (VRampControl_->On))
  {
    if(VRampControl_->RampDACCounter++ > MsTicksOnTime)
    {
      DAC1_Load10bitInputData(0);
      StopPSU();

      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 0;
      VRampControl_->On = 0;
    }
  }
  
  //all LEDs off 
  if ((!VRampControl_->Ramp) && (!VRampControl_->On))
  {
    if(VRampControl_->RampDACCounter++ > MsTicksOffTime)
    {
      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 1;
      VRampControl_->On = 0;
      
      VRampControl_->GlobalTestCounter++;

      VRampControl_->Start = 1;

    } 
  }
}
//------------------------------------------------------------------------------
// Ramp down
//    ----------------                                    -----------
//  |                 -                                  |  
//  |                   -                                |
//  |                     -                              |
//  |                       -                            |
//  |                         -                          |
//  |                           -                        |
//  |                             -                      |
//  |                               -                    |
//  |                                 -                  |
//  |                                   -                |
//  <-----------------><-----------------><-------------->
//     (.Ramp !.On )     (.On  !.Ramp)     (!.Ramp  !.On)
//------------------------------------------------------------------------------
void RampControllRampDown (RampControl_t *VRampControl_)
{
  uint8_t i_;
  
  //ramp down
//all LEDs on with small steps from min to max
  if ((VRampControl_->Ramp) && (!VRampControl_->On))
  {
    if(VRampControl_->Start)
    {
      StartPSU();
      VRampControl_->Start = 0;
      VRampControl.VoutputVoltage = 0;
      VRampControl.RisingSteps = (uint16_t)RampRisingEdge;
      
      for(i_=0;i_<200;i_++)
      {
        DAC1_Load10bitInputData(RampDACValues[i_]);
      }  
    } 
    

    if (VRampControl_->RampDACCounter < MsTicksOnTime)
    {
//////      if(VRampControl_->RampDACCounter < 11)
////      if(VRampControl_->RampDACCounter < RisingEdgeCount+1)
////      {  
////        VRampControl_->VoutputVoltage = VRampControl_->VoutputVoltage + VRampControl.RisingSteps;
////      }  
////      else
    
////      {
////        VRampControl.VoutputVoltage = RampEndVoltage;
////      }
////      DAC1_Load10bitInputData(VRampControl_->VoutputVoltage);
      VRampControl_->RampDACCounter++;
    }
    else 
    {
      VRampControl_->RampDACCounter = 0;
      VRampControl_->Ramp = 0;
      VRampControl_->On = 1;
    }    
  }
  
  
  //all LEDs on after ramp up
  if ((!VRampControl_->Ramp) && (VRampControl_->On))
  {
    if (VRampControl_->RampDACCounter < MsTicksRampTime)
    {
////      VRampControl_->VoutputVoltage = VRampControl_->VoutputVoltage - VRampControl_->RisingSteps;
      VRampControl_->VoutputVoltage = RampDACValues[199-VRampControl_->RampDACCounter];
      DAC1_Load10bitInputData(VRampControl_->VoutputVoltage);
          
      VRampControl_->RampDACCounter++;
    }
    else 
    {
     DAC1_Load10bitInputData(0);
     StopPSU();
    
     VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 0;
      VRampControl_->On = 0;
    }    
  }
  
  //all LEDs off 
  if ((!VRampControl_->Ramp) && (!VRampControl_->On))
  {
    if(VRampControl_->RampDACCounter++ > MsTicksOffTime)
    {
      VRampControl_->RampDACCounter = 0;

      VRampControl_->Ramp = 1;
      VRampControl_->On = 0;
      
      VRampControl_->GlobalTestCounter++;

      VRampControl_->Start = 1;

    } 
  }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
