/* *****************************************************************************************************
 * SOFTWARE COPYRIGHT NOTICE 
 *  
 * (c) 2016 Microchip Technology Inc. and its subsidiaries.  
 * You may use this software and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR 
 * STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 * MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, 
 * COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 * 
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL OR CONSEQUENTIAL 
 * LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN 
 * IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST 
 * EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR 
 * THIS SOFTWARE.
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS. 
 * ****************************************************************************************************/


#include <xc.h>
#include <stdint.h>

#include "globals.h"
#include "mcc.h"
//------------------------------------------------------------------------------
//Fault Handler Function
//------------------------------------------------------------------------------

void FaultHandler ( void )
{

	if ( !fault_controller.status.flt_fault_detect && ( !sepic.status.global_fault ) )
	{
		return;
	}
	else if ( ( !fault_controller.status.flt_fault_detect ) && ( sepic.status.global_fault ) )
	{

		NOP();

		fault_controller.fault_recovery_counter++;

		if ( fault_controller.fault_recovery_counter >= fault_controller.fault_recovery_counter_thshld )
		{
			fault_controller.fault_recovery_counter = 0;

			fault_controller.status.flt_uvlo_shutdown = 0;
			fault_controller.status.flt_ovlo_shutdown = 0;
			fault_controller.status.flt_ovp_shutdown = 0;
			fault_controller.status.flt_ot_warning = 0;
			fault_controller.status.flt_otp_shutdown = 0;
			fault_controller.status.flt_open_circuit = 0;
			sepic.status.global_fault = 0;
			sepic.status.enabled = 1;

			FaultRecovery();

		}
	}
	else if ( ( fault_controller.status.flt_fault_detect ) && ( !sepic.status.global_fault ) )
	{
		// New fault has been detected => Shut down converter if it hasn't been already

		if(sepic.status.enabled) FaultShutDown();

		NOP(); // Place breakpoint here during debugging

	}
	else
	{
		NOP();
	}

	
	// Reset temporary fault 
	fault_controller.status.flt_fault_detect = 0;
	
	
	return;

}

//------------------------------------------------------------------------------
// FaultShutDown
//------------------------------------------------------------------------------

void FaultShutDown ( void )
{
	ErrorPin_WR = 1; // Set diagnostic output HIGH

	T2CONbits.TMR2ON = 0; //CCP
	COG1CON0bits.G1EN = 0; //COG1
	CCP1CONbits.EN = 0; // Disable PWM module
//	PWM5CONbits.EN = 0; // Disable dimming PWM

  OPA1CONbits.EN = 0; //FTx RC2/OPAout issue solved (see ppt)
	
//  PWM1_DutyValueSet(0x00); // Set max duty cycle to 0%

	sepic.status.global_fault = 1; // Set global fault flag
	sepic.status.enabled = 0; // Clear global enable bit
	
	return;
}


//------------------------------------------------------------------------------
// FaultRecovery
//------------------------------------------------------------------------------

void FaultRecovery ( void )
{
//	PWM1_DutyValueSet(tCCP1DutyCycleClamping); // Set max duty cycle to ~90%
//////
//  OPA1CONbits.EN = 1; //FTx RC2/OPAout issue solved (see ppt)
//  
////	PWM5CONbits.EN = 1; // Enable dimming PWM
	CCP1CONbits.EN = 1; // Enable PWM module
//	COG1CON0bits.G1EN = 1; // Enable switching frequency PWM output (COG1)
//	T2CONbits.TMR2ON = 1; // Enable switching frequency PWM time base (CCP)

//  VRampControl.Start = 1; //add FTx
//  VRampControl.RampDACCounter = 0;
//  
//  
//  VRampControl.Ramp = 1;  
//  VRampControl.Start = 1;
//  VRampControl.On = 0;
  
//  VRampControl.RisingSteps = (uint8_t)RampRisingStep;
//  VRampControl.FunctionMode = 0;
    
	ErrorPin_WR = 0; // Set diagnostic output LOW
	
	return;
}

//------------------------------------------------------------------------------
// Fault Handler Initialization
//------------------------------------------------------------------------------

void InitFaultHandler ( void )
{
	//Fault Consideration Init
	// fault Handler is set up with all faults active and fault flags set
	// These have to be cleared by the fault handler before the converter is launched

	/* Over Voltage Lock Out Fault Settings */
	fault_controller.fltSettingsOVLO.threshold = fltOVLOCounterThshld; // Set fault counter threshold
	fault_controller.fltSettingsOVLO.counter = fltOVLOCounterThshld + 1; // Set fault counter 
	fault_controller.fltSettingsOVLO.trip = fltOVLOTrip; // Set fault trip point threshold
	fault_controller.fltSettingsOVLO.release = fltOVLORelease; // Set fault release point threshold
	fault_controller.status.flt_ovlo_shutdown = 0;//FTx 1; // Set Flag bit during startup

	/* Under Voltage Lock Out Fault Settings */
	fault_controller.fltSettingsUVLO.threshold = fltUVLOCounterThshld; // Set fault counter threshold
	fault_controller.fltSettingsUVLO.counter = fltUVLOCounterThshld + 1; // Set fault counter 
	fault_controller.fltSettingsUVLO.trip = fltUVLOTrip; // Set fault trip point threshold
	fault_controller.fltSettingsUVLO.release = fltUVLORelease; // Set fault release point threshold
	fault_controller.status.flt_uvlo_shutdown = 0;//FTx 1; // Set Flag bit during startup

	/* Output Over Voltage Fault Settings */
	fault_controller.fltSettingsOVP.threshold = fltOVPCounterThshld; // Set fault counter threshold
	fault_controller.fltSettingsOVP.counter = fltOVPCounterThshld + 1; // Set fault counter 
	fault_controller.fltSettingsOVP.trip = fltOVPTrip; // Set fault trip point threshold
	fault_controller.fltSettingsOVP.release = fltOVPRelease; // Set fault release point threshold
	fault_controller.status.flt_ovp_shutdown = 0;//FTx 1; // Reset Flag bit during startup

	/* LED Temperature Warning Settings */
	fault_controller.fltSettingsOTW.threshold = fltOTWCounterThshld; // Set fault counter threshold
	fault_controller.fltSettingsOTW.counter = fltOTWCounterThshld + 1; // Set fault counter 
	fault_controller.fltSettingsOTW.trip = fltOTWTrip; // Set fault trip point threshold
	fault_controller.fltSettingsOTW.release = fltOTWRelease; // Set fault release point threshold
	fault_controller.status.flt_ot_warning = 0;//FTx 1; // Set Flag bit during startup

	/* LED Temperature Protection Settings */
	fault_controller.fltSettingsOTP.threshold = fltOTPCounterThshld; // Set fault counter threshold
	fault_controller.fltSettingsOTP.counter = fltOTPCounterThshld + 1; // Set fault counter 
	fault_controller.fltSettingsOTP.trip = fltOTPTrip; // Set fault trip point threshold
	fault_controller.fltSettingsOTP.release = fltOTPRelease; // Set fault release point threshold
	fault_controller.status.flt_otp_shutdown = 0;//FTx 1; // Set Flag bit during startup

	fault_controller.fault_recovery_counter_thshld = fltRecoveryCounterThshld;
	fault_controller.fault_recovery_counter = 0;
	fault_controller.status.flt_fault_detect = 0;
	
	sepic.status.enabled = 0; // Converter is disabled
	sepic.status.global_fault = 0;//FTx 1; // Global Fault Flag is set

	return;

}


