/* *****************************************************************************************************
 * SOFTWARE COPYRIGHT NOTICE 
 *  
 * (c) 2016 Microchip Technology Inc. and its subsidiaries.  
 * You may use this software and any derivatives exclusively with Microchip products. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR 
 * STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 * MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP PRODUCTS, 
 * COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
 * 
 * IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL OR CONSEQUENTIAL 
 * LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN 
 * IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST 
 * EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR 
 * THIS SOFTWARE.
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE TERMS. 
 * ****************************************************************************************************/


#include <xc.h>
#include <stdint.h>
#include "globals.h"

#include "mcc.h"
//------------------------------------------------------------------------------
// Check operation mode status and switch op mode if needed
//------------------------------------------------------------------------------

////void task_CheckOperationModeStatus ( void )
////{
////
////	switch ( task_mgr.op_mode )
////	{
////	case OP_MODE_STARTUP:
////		// Switch to normal operation if start-up self-test is completed and no fault
////		// condition has been detected
////		if ( ( sepic.status.startup ) && ( !sepic.status.global_fault ) )
////		{
////			task_mgr.op_mode = OP_MODE_NORMAL;
////			task_mgr.task_flow = task_flow_def_normal;
////			task_mgr.task_tick_index = 0;
////			sepic.led_forward_current = sepic.led_bin_current;
////		}
////		break;
////
////	case OP_MODE_NORMAL:
////		// Switch to Fault Mode when a global fault has been detected
////		if ( sepic.status.global_fault )
////		{
////			task_mgr.op_mode = OP_MODE_FAULT;
////			task_mgr.task_flow = task_flow_def_fault;
////			task_mgr.task_tick_index = 0;
////		}
////		break;
////
////	case OP_MODE_FAULT:
////		// Switch to normal operation if fault condition has been resolved
////		if ( !sepic.status.global_fault )
////		{
////			task_mgr.op_mode = OP_MODE_NORMAL;
////			task_mgr.task_flow = task_flow_def_normal;
////			task_mgr.task_tick_index = 0;
////			sepic.status.startup = 0;
////			sepic.status.phase_in = 0;	// No phase-in/soft-start from fault recovery
////		}
////		break;
////    
////  case OP_MODE_INIT:  
////   	task_mgr.op_mode = OP_MODE_STARTUP;
////		task_mgr.task_flow = task_flow_def_startup;
////		task_mgr.task_tick_index = 0;
////		sepic.status.startup = 1;
////		
////		sepic.status.phase_in = USE_SOFTSTART;
////		#if (USE_SOFTSTART == 1)
////		sepic.led_forward_current = startupLEDCurrent;
////		#else
////		sepic.led_forward_current = sepic.led_bin_current;
////		#endif
////
////    break;
////    
////	default:
////////		task_mgr.op_mode = OP_MODE_STARTUP;
////////		task_mgr.task_flow = task_flow_def_startup;
////////		task_mgr.task_tick_index = 0;
////////		sepic.status.startup = 1;
////////		
////////		sepic.status.phase_in = USE_SOFTSTART;
////////		#if (USE_SOFTSTART == 1)
////////		sepic.led_forward_current = startupLEDCurrent;
////////		#else
////////		sepic.led_forward_current = sepic.led_bin_current;
////////		#endif
////
////		break;
////	}
////
////	return;
////}
//------------------------------------------------------------------------------
// Idle Task = Do nothing
//------------------------------------------------------------------------------

////void task_Idle ( void )
////{
////	/* Do Nothing*/
////	return;
////}

//------------------------------------------------------------------------------
// Input under voltage data acquisition & test
//------------------------------------------------------------------------------

void task_CheckInputUnderVoltage ( void )
{

	// get most recent input voltage
	sepic.input_voltage = ADC_GetConversion(adinInputVoltage);
  UARTsend.send[1] = (uint8_t)(sepic.input_voltage >> 2);

	// Check for Under Voltage Lock Out (UVLO) conditions
	if ( sepic.input_voltage <= fault_controller.fltSettingsUVLO.trip )
	{
		fault_controller.fltSettingsUVLO.counter++; // Increment counter value
	}
	else if ( sepic.input_voltage > fault_controller.fltSettingsUVLO.release )
	{
		fault_controller.fltSettingsUVLO.counter = 0; // Reset counter value
	}

	if ( fault_controller.fltSettingsUVLO.counter >= fault_controller.fltSettingsUVLO.threshold ) // Check fault counter
	{
		fault_controller.fltSettingsUVLO.counter = fault_controller.fltSettingsUVLO.threshold; // Clamp counter value
		fault_controller.status.flt_uvlo_shutdown = 1; // Set fault flag
		fault_controller.status.flt_fault_detect = 1;

		FaultShutDown();	// switch off converter immediately
	}

	return;
	
}
//------------------------------------------------------------------------------
// Input over voltage data acquisition & test
//------------------------------------------------------------------------------

void task_CheckInputOverVoltage ( void )
{
	
	// Check for Over Voltage Lock Out (OVLO) conditions
	if ( sepic.input_voltage >= fault_controller.fltSettingsOVLO.trip )
	{
		fault_controller.fltSettingsOVLO.counter++; // Increment counter value
	}
	else if ( sepic.input_voltage < fault_controller.fltSettingsOVLO.release )
	{
		fault_controller.fltSettingsOVLO.counter = 0; // Reset counter value
	}

	if ( fault_controller.fltSettingsOVLO.counter >= fault_controller.fltSettingsOVLO.threshold ) // Check fault counter
	{
		fault_controller.fltSettingsOVLO.counter = fault_controller.fltSettingsOVLO.threshold; // Clamp counter value
		fault_controller.status.flt_ovlo_shutdown = 1; // Set fault flag
		fault_controller.status.flt_fault_detect = 1;

		FaultShutDown();	// switch off converter immediately
	}

	return;

}

//------------------------------------------------------------------------------
// Over temperature warning data acquisition & test
//------------------------------------------------------------------------------

void task_CheckTemperatureWarning ( void )
{
////	uint16_t i_read = 0;
////
////	i_read = ADC_GetConversion(adinTemperature); // Read NTC voltage
////	sepic.led_temperature = tab_ntc[i_read];
////  UARTsend.send[2] = sepic.led_temperature;
////
////
////	// Check for Over Temperature Warning conditions
////	if ( sepic.led_temperature >= fault_controller.fltSettingsOTW.trip )
////	{
////		fault_controller.fltSettingsOTW.counter++; // Increment counter value
////	}
////	else if ( sepic.led_temperature < fault_controller.fltSettingsOTW.release )
////	{
////		fault_controller.fltSettingsOTW.counter = 0; // Reset counter value
////	}
////
////	if ( fault_controller.fltSettingsOTW.counter >= fault_controller.fltSettingsOTW.threshold ) // Check fault counter
////	{
////		fault_controller.fltSettingsOTW.counter = fault_controller.fltSettingsOTW.threshold; // Clamp counter value
////		fault_controller.status.flt_ot_warning = 1; // Set fault flag
////		sepic.status.temp_derating_active = 1;
////	}
////	else
////	{
////		sepic.status.temp_derating_active = 0;
////	}
////
////	return;
}

//------------------------------------------------------------------------------
// Over temperature protection data acquisition & test
//------------------------------------------------------------------------------

////void task_CheckTemperatureProtection ( void )
////{
////#if (SILICON_REV != SILICON_REV_A1)
////	
////	// Check for Over Temperature Protection conditions
////	if ( sepic.led_temperature >= fault_controller.fltSettingsOTP.trip )
////	{
////		fault_controller.fltSettingsOTP.counter++; // Increment counter value
////	}
////	else if ( sepic.led_temperature < fault_controller.fltSettingsOTP.release )
////	{
////		fault_controller.fltSettingsOTP.counter = 0; // Reset counter value
////	}
////
////	if ( fault_controller.fltSettingsOTP.counter >= fault_controller.fltSettingsOTP.threshold ) // Check fault counter
////	{
////		fault_controller.fltSettingsOTP.counter = fault_controller.fltSettingsOTP.threshold; // Clamp counter value
////		fault_controller.status.flt_otp_shutdown = 1; // Set fault flag
////		fault_controller.status.flt_fault_detect = 1;
////
////		sepic.status.global_fault = 1;	// Set global fault flag
////		FaultShutDown();	// switch off converter immediately
////	}
////
////	// Check for broken cable connection between LED driver and load
////	if ( sepic.led_temperature == -100 )
////	{
////		fault_controller.status.flt_open_circuit = 1;
////		fault_controller.status.flt_fault_detect = 1;
////
////		FaultShutDown();	// switch off converter immediately
////	}
////#endif
////
////
////	return;
////}
//------------------------------------------------------------------------------
// check switch pressed for Dimming Value Set
//------------------------------------------------------------------------------

void task_CheckSwitch ( void )
{
//#ifndef __DEBUG

	if ( !SwitchInput_RD )
	{
		sw_controller.switch_pre_state = 0; // Key is pressed
	}
	else
	{
		if ( !sw_controller.switch_pre_state )
		{
			sw_controller.switch_counter++;
		}
		else
		{
			sw_controller.switch_counter = 0;
		}
	}

	if ( sw_controller.switch_counter > sw_controller.switch_counter_threshold )
	{		
    sw_controller.switch_pre_state = 1;
		sw_controller.switch_counter = sw_controller.switch_counter_threshold;

    if((VRampControl.FunctionMode == 2) || (VRampControl.FunctionMode == 1)
                                        || (VRampControl.FunctionMode == 0))
    {
      VRampControl.FunctionModeSet = 3;
    }  
    
    if((VRampControl.FunctionMode == 3) && (VRampControl.FunctionModePrevious == 2))
    {
      VRampControl.FunctionModeSet = 0;
      VRampControl.FunctionModePrevious = 0;
    }  
    else if((VRampControl.FunctionMode == 3) && (VRampControl.FunctionModePrevious == 1))
    {
      VRampControl.FunctionModeSet = 2;
      VRampControl.FunctionModePrevious = 2;
    }  
    else if((VRampControl.FunctionMode == 3) && (VRampControl.FunctionModePrevious == 0))
    {
      VRampControl.FunctionModeSet = 1;
      VRampControl.FunctionModePrevious = 1;
    }  

	}

//#endif


	return;
}
//------------------------------------------------------------------------------
// Set new dimming value
//------------------------------------------------------------------------------

void task_SetDimmingDutyRatio ( void )
{
  // Dimming value clamping
////	if ( sepic.led_dim_ratio >= regDimmingPWMPeriod )
////	{
////		sepic.led_dim_ratio = regDimmingPWMPeriod + 1;
////	}
////
////	// Set new dimming value
////
////	if ( regDimmingPWMDutyCycle != sepic.led_dim_ratio )
////	{
////		regDimmingPWMDutyCycle = sepic.led_dim_ratio;
////		regDimmingPWM_DC_LDA = 1;
////	}
 
	return;
}

//------------------------------------------------------------------------------
// Set new dimming value
//------------------------------------------------------------------------------

////void task_SetForwardCurrentRefrence ( void )
////{
////	uint16_t reg_dum = 0;
////
////	// Read recent DAC value
////  reg_dum = DAC1REF;
////	
////	// Skip current reference adjustment
////	if ( ( reg_dum == sepic.led_forward_current ) || ( sepic.status.global_fault ) )
////	{
////		return;							 // No change required => Exit 
////	}
////	else if(!sepic.status.phase_in)
////	{
////    DAC1_Load10bitInputData(sepic.led_forward_current);
////	}
////	else if(sepic.status.phase_in)		// Set new dimming value
////	{
////////		if(reg_dum < startupLEDCurrent)
////////		{
////////			reg_dum = startupLEDCurrent;	// Check DAC value for lower boundary
////////		} 
////////		
////////		// Increment current reference for soft transitions
////////		if ( reg_dum < sepic.led_forward_current )
////////		{
////////      DAC1_Load10bitInputData(++reg_dum);
////////		}
////////		else if ( reg_dum > sepic.led_forward_current )
////////		{
////////      DAC1_Load10bitInputData(--reg_dum);
////////		}
////////		else
////////		{
////////      DAC1_Load10bitInputData(reg_dum);
////////		}
////	}
////	else
////	{
////	}
////	
////
////	return;
////}

//------------------------------------------------------------------------------
// Output voltage data acquisition & test
//------------------------------------------------------------------------------

void task_CheckOutputVoltage ( void )
{
	sepic.output_voltage = ADC_GetConversion(adinOutputVoltage);
  UARTsend.send[3] = (uint8_t)(sepic.output_voltage >> 2);

	// Check for Hardware Output Over Voltage Shut Down (OVP)
	if ( fault_controller.status.flt_ovp_shutdown )
	{
		// shut Down is performed in the comparator ISR
		fault_controller.status.flt_fault_detect = 1;
	}

	return;
}
//------------------------------------------------------------------------------
// LED forward current data acquisition & test
//------------------------------------------------------------------------------

////void task_CheckForwardCurrent ( void )
////{
////#ifndef __DEBUG
////	// TODO: Set-up ADC input for IFWD sampling
////	sepic.output_current = ADC_GetConversion(adinForwardCurrent);
////#endif
////
////	return;
////}
//------------------------------------------------------------------------------
// BIN resistor measurement and BIN class detection
//------------------------------------------------------------------------------

////void GetBIN ( void )
////{
////
////#if (USE_BINNING == 1)
////	// Auto-Binning enabled
////
////	uint16_t binr = 0, i = 0;
////	;
////
////	if ( sepic.led_bin_class == 0 ) // Only call BIN detection once
////	{
////
////		binr = ADC_GetConversion(adinBinResistor);
////		binr &= bin_class_feedback_mask;
////
////		for ( i = bin_class_feedback_ubound; i > bin_class_feedback_lbound; i-- )
////		{
////			if ( tab_bin_class_feedback[i] <= binr )
////			{
////				// Set BIN class and exit loop 
////
//// 				sepic.led_bin_current = tab_bin[i];
////				sepic.led_bin_class = ( i + 1 );
////
////				break;
////			}
////		}
////
////	}
////  
////#else 
////	// Auto-Binning disabled
////	if ( sepic.led_bin_class == 0 ) // Only call BIN detection once
////	{
////		sepic.led_bin_current = tab_bin[bin_class_nominal];
////		sepic.led_bin_class = bin_class_nominal;
////
////	}
////#endif
////
////
////	return;
////}

//------------------------------------------------------------------------------
// UART receive handler
// 
// 
// 
// 
// 
// 
// 
// 
// 
// 
//------------------------------------------------------------------------------
unsigned char RecBytes[4], RecByteCounter = 0;
volatile struct UARTHandle_
{
  unsigned RecFrameReady : 1;
  unsigned : 1;
  unsigned : 1;
  unsigned : 1;
  unsigned : 1;
  unsigned : 1;
  unsigned : 1;
  unsigned : 1;
} UARTHandle;

void task_RecUart(void)
{
  if (PIR1bits.RCIF)
  {
    RecBytes[RecByteCounter++] = EUSART_Read();
    PIR1bits.RCIF = 0;
    if (RecBytes[3] == 0x0D)
    {
      RecByteCounter = 0;
      UARTHandle.RecFrameReady = 1;
      RecBytes[3] = 0x00;
    }
    else
    {
      UARTHandle.RecFrameReady = 0;
    }
  }
  if (UARTHandle.RecFrameReady)
  {
    UARTHandle.RecFrameReady = 0;
    if (RecBytes[0] == 0xA1)
    {
      sepic.led_forward_current = (uint16_t) (RecBytes[1] << 8) | RecBytes[2];
      DAC2_Load10bitInputData(sepic.led_forward_current);
    }

    if (RecBytes[0] == 0xA3)
    {
      VRampControl.FunctionModeSet = (uint16_t) (RecBytes[1] << 8) | RecBytes[2];
    }
  }
  
  return;
}
//------------------------------------------------------------------------------
// UART transmitt handler
// UARTsend array[0] = 0xAA
// UARTsend array[1] = input Voltage
// UARTsend array[2] = temperature
// UARTsend array[3] = output voltage
// UARTsend array[4] = sepic.status
// UARTsend array[5] = fault_controller.status
// UARTsend array[6] = dimming ratio
// UARTsend array[7] = BIN resistor
// UARTsend array[8] = LED forward current
// UARTsend array[9] = 0x0D
//------------------------------------------------------------------------------
void task_SendUart ( void )
{
//  DebugPin_WR = 1;
  UARTsend.send[4] = sepic.status.status;
  UARTsend.send[5] = fault_controller.status.status;

  if(TXSTAbits.TRMT)
  {
    EUSART_Write(UARTsend.send[UARTsend.counter++]);
  }

  if(UARTsend.counter > UARTsend.maxCount)
  {
    UARTsend.counter = 0;
  }
  
//  DebugPin_WR = 0;
	
  return;
}

//------------------------------------------------------------------------------
// Animation 
//------------------------------------------------------------------------------

void task_Animation (void)
{
  if(VRampControl.FunctionMode != VRampControl.FunctionModeSet)
  {
    //---------------
    if((VRampControl.FunctionModeSet == 3) && (VRampControl.FunctionMode == 0))
    {
      if(!VRampControl.Start)
      {  
        RampControll(&VRampControl);
      }
      else
      {
        VRampControl.FunctionMode = VRampControl.FunctionModeSet;
      }
    }
    if((VRampControl.FunctionModeSet == 0) && (VRampControl.FunctionMode == 3))
    {
      VRampControl.FunctionMode = VRampControl.FunctionModeSet;
    }
    //----------------
    if((VRampControl.FunctionModeSet == 3) && (VRampControl.FunctionMode == 1))
    {
      if(!VRampControl.Start)
      {  
        HazardControll(&VRampControl);
      }
      else
      {
        VRampControl.FunctionMode = VRampControl.FunctionModeSet;
      }
    }
    if((VRampControl.FunctionModeSet == 1) && (VRampControl.FunctionMode == 3))
    {
      VRampControl.FunctionMode = VRampControl.FunctionModeSet;
    }
    //---------------
    if((VRampControl.FunctionModeSet == 3) && (VRampControl.FunctionMode == 2))
    {
      if(!VRampControl.Start)
      {  
        RampControllRampDown(&VRampControl);
      }
      else
      {
        VRampControl.FunctionMode = VRampControl.FunctionModeSet;
      }
    }
    if((VRampControl.FunctionModeSet == 2) && (VRampControl.FunctionMode == 3))
    {
      VRampControl.FunctionMode = VRampControl.FunctionModeSet;
    }
  }  
  
  else if(VRampControl.FunctionMode == VRampControl.FunctionModeSet)
  {  
    switch(VRampControl.FunctionMode)
    {  
      case 0:
        RampControll(&VRampControl);
      break;
    
      case 1:
        HazardControll(&VRampControl);
      break;  
    
      case 2:
        RampControllRampDown(&VRampControl);
      break;
    
      case 3:
      
      break;  
    
      default:
      break;  
    }
  }
/*
//  RampControll(&VRampControl);  //14us
  
  if((VRampControl.BlinkOn) || (!VRampControl.BlinkOn && !VRampControl.ScenarioReady))
  {  
    //-------------------------------------------------
    if(VRampControl.RampPause)  //pause after complete simulation
    {
////      RampControllWait(&VRampControl);
    }
    else if(!VRampControl.HazardLight)  //running light up or down
    {  
      if(!VRampControl.RampDown)
      {
        RampControll(&VRampControl);  //14us
      }
      else
      {
        RampControllRampDown(&VRampControl);  
      }
    }
    else  //hazard light
    {
      HazardControll(&VRampControl);
    }
    //-------------------------------------------------
    if(VRampControl.GlobalTestCounter > 9) //10 times running light up
    {
      VRampControl.HazardLight = 1;
    }      
      
    if(VRampControl.GlobalTestCounter > 19) //10 times hazard light
    {
      VRampControl.HazardLight = 0;
      VRampControl.RampDown = 1;
    }
    if(VRampControl.GlobalTestCounter > 29) //10 times running light down
    {
      VRampControl.RampDown = 0;
      VRampControl.RampPause = 1;
    }  
    if(VRampControl.GlobalTestCounter > 31) //2 times pause
    {
      VRampControl.RampPause = 0;
      VRampControl.GlobalTestCounter = 0;
    }
  }*/
}
// ======================================================================================================
// Basic Task Scheduler Structure Initialization
// ======================================================================================================

////void InitScheduler ( void )
////{
////	task_mgr.op_mode = 0;
////	task_mgr.task_id = 0;
////	task_mgr.task_tick_index = 0;
////
////	return;
////}

void InitSwitchController ( void )
{
	sw_controller.switch_counter = 0;
	sw_controller.switch_counter_threshold = 20;
	sw_controller.switch_pre_state = 1;
	return;
}

//------------------------------------------------------------------------------
// Initialize UART send chain
//------------------------------------------------------------------------------

void InitUARTsend (void)
{
  uint8_t i_;

  UARTsend.maxCount = 9;
  UARTsend.send[0] = 0xAA;
  UARTsend.send[UARTsend.maxCount] = 0x0D;
  UARTsend.SendTaskCounter = 0;
  
  UARTsend.counter = 0;
 
  for(i_=1; i_< UARTsend.maxCount; i_++)
  {  
    UARTsend.send[i_] = i_;  
  }
  
  return;
}
//------------------------------------------------------------------------------
// Initialize Ramp Control
//------------------------------------------------------------------------------

void InitRampControl (void)
{
//  uint8_t i_;
//  
//  for(i_=0;i_<200;i_++)
//  {
//    RampDACValues[i_] = i_;
//  }  
  VRampControl.Ramp = 1;  
  VRampControl.Start = 1;
  VRampControl.BlinkOn = 1;
  
////  VRampControl.RisingSteps = (uint8_t)RampRisingStep;
  VRampControl.FunctionMode = 2;
  VRampControl.FunctionModePrevious = 2;
  VRampControl.FunctionModeSet = 2;
  
//  DAC2_Load10bitInputData(223); //15LEDs
  DAC2_Load10bitInputData(210); //16LEDs
  
  return;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
